% Bokor_FaultDetEx.m
%
% This file computes a robust fault detection filter for the
% example problem posed by Jozsef.
%
% PJS 7/14/2010  Initial Coding

%------------------------------------------------------------------------
% Problem Data (Section 5 of reference)
% Create the uncertain plant Pu(Delta) such that
%   [y;u;z] = Pu(Delta)*[f(2); u(2); n(2)]
% where z = Fref*f
%------------------------------------------------------------------------
d1 = ureal('d1',0,'PlusMinus',0.1);
d2 = ureal('d2',0,'PlusMinus',0.1);
A = [-0.8 0 0; ...
    0 -0.5*(1+d1) 0.6*(1+d2); 
    0  -0.6*(1+d2) -0.5*(1+d1)]; 
B = [1 1; 1 0; 0 1];
C = [0 1 1;1 1 0];

nx = 3;

Bbar = [B B zeros(nx,2)];
Cbar = [C;zeros(2,nx)];
Dbar = [zeros(2) zeros(2) 0.001*eye(2); ...
        zeros(2) eye(2) zeros(2)];

% Construct [y;u] = Gu(Delta)*[f(2); u(2); n(2)]
% without reference model
if 1
    % Parametric Uncertainty only
    Gu = ss(A,Bbar,Cbar,Dbar);
else
    % Input unmodeled dynamics only
    Gu = ss(A.NominalValue,Bbar,Cbar,Dbar);
    Wt = makeweight(0.1,1,2); % *0.5;
    DEL1 = ultidyn('DEL1',[2 2],'Type','GainBounded','Bound',1);
    DEL2 = ultidyn('DEL2',[2 2],'Type','GainBounded','Bound',1);
    Gu = Gu*blkdiag(eye(2)+Wt*DEL1,eye(2)+Wt*DEL2,eye(2));
end

% Construct [y;u;z] = Pu(Delta)*[f(2); u(2); n(2)]
% with reference model z = Fref*f
FREFCASE = 2;
switch FREFCASE
    case 1
        % Simple Identity Reference
        Fref = ss([eye(2) zeros(2)]);
    case 2
        % From Andras Varga (SysTol Paper on H2 fault detection)
        AQ = [-9.7843 -0.6691 0.6115 4.1286 0.0231; ...
              0.0507 -0.8519 -0.7203 7.9819 -0.1187; ...
              0.0112 2.1889 -10.4625 1.8786 -0.0438; ...
              -0.0109 -0.1273 0.0018 -10.3287 0.0019; ...
              -0.0148 -1.1154 5.3290 -0.9532 -9.7062];
          
        BQ = [1.6954 0.0088 -0.0162 -0.0447; ...
              3.8998 -1.1776 0.2226 0.7654; ...
              2.3304 -5.0625 -0.7868 -0.1619; ...
            -4.9479 1.1771 -0.2928 -0.3650; ...
            -1.2682 2.5215 -0.0421 -0.0120];
        
        CQ = [-69.526 24.805 -136.988 54.786 257.633; ...
              244.302 6.003 35.909 -112.301 -93.808];

        DQ = [45.2108 -66.3827 0 0; ...
              -54.0324 24.0389 0 0];

        Fh2 = ss(AQ,BQ,CQ,DQ);
         
        % Design reference model used in Varga's paper
        M11 = tf([1 -1.1336],[1 20 100]);
        %M11 = M11/norm(M11,inf);
        M11 = M11/evalfr(M11,0);
        M = [M11 0; 0 M11];
        
        R = Fh2*Gu.Nominal;        
        Fref = R(:,1:4);
    case 3
        % Geometric Filter from  Jozsef (7/16/2010)
        %A1 = [-1.1 0.764; -0.6364 -0.4119];
        %B1y = -[-0.0573 -0.5427; 0.0623 -0.2795];
        A1 = [-1.1 0.4823; -0.6364 -3.896];
        B1y = [0.259 0.341; 2.4013 -2.1892];
        B1u = [1.0 0; -0.707 0];
        C1 = [0 1];
        D1y = -[0.707 -0.707];
        D1u = [0 0];
        Fgeo1 = ss(A1,[B1y B1u],C1,[D1y D1u]);

        %A2 = [-0.559 0.5529; -0.4413 -0.5241];
        %B2y = -[0.1287 -0.1704; -0.241 0.3121];
        A2 = [0.3896 1.8945; -5.8863 -8.2245];
        B2y = [-1.4702 0.8412; 7.7295 -4.1622];
        B2u = [0 -0.707; 0 1];
        C2 = [0.5774 .8165];
        D2y = -[0.8165 -0.4082];
        D2u = [0 0];
        Fgeo2 = ss(A2,[B2y B2u],C2,[D2y D2u]);

        Fgeo = [Fgeo1; Fgeo2];

        % Normalize gain
        R = Fgeo*Gu.Nominal;    
        Fgeo1 = Fgeo1/evalfr(R(1,1),0);
        Fgeo2 = Fgeo2/evalfr(R(2,2),0);
        Fgeo = [Fgeo1; Fgeo2];

        % Test Fault Decoupling of Geometric Filter 
        Ru = Fgeo*Gu; 
        figure;
        step(Ru(:,1));
        figure;
        step(Ru(:,2));

        % Simulate Fault Decoupling of Geometric Filter with sinusoid
        TT = linspace(0,20,1e3)';
        omega0 = 0.64;
        UU = sin(omega0*TT);
        figure;
        lsim(usample(Ru(:,1),10),UU,TT);
        figure;
        lsim(usample(Ru(:,2),10),UU,TT);
        drawnow;
        
        % Bode Mag plots 
        %figure; tmp = Ru(1,:); bodemag(tmp,'b',tmp.Nominal,'r--')
        %figure; tmp = Ru(2,:); bodemag(tmp,'b',tmp.Nominal,'r--')
        
        % Construct Reference Model
        % In a discussion with Gary we concluded that the reference
        % model needs to include the inputs u as well as the faults f.
        R = Fgeo*Gu.Nominal;        
        Fref = R(:,1:4);
        
        %Fref = ss([eye(2) zeros(2)]);
end
%return

Pu = [Gu; Fref zeros(2,2)];
Nz = 2; Nw = 4; Nd = 6; Ny = 4;

%------------------------------------------------------------------------
% Use freq. gridding to compute lower bound on optimal L2-L2 gain 
%------------------------------------------------------------------------
nomega = 100; %250;
omega = logspace(-1,1,nomega);
nmeas = Ny;
tic
[gain1,Fopt1,PIopt1] = robfiltlb(Pu,nmeas,omega);
t1=toc;
[gain1lb,mxidx1]=max(gain1);
fprintf('Freq. Grid: Lower Bound on minimal L2-L2 gain = %4.3f\n',gain1lb);
fprintf('Comp time = %4.3f sec\n',t1);

% Plot lower bnd vs. frequency
fh=figure;
ph=semilogx(omega,gain1);
xl=xlabel('Frequency (rad/sec)');
yl=ylabel('Lower Bound');
tl = title('Performance Lower Bound vs. Frequency');
grid on;
set([xl yl tl gca],'FontSize',18);
set(ph,'LineWidth',3);

switch FREFCASE
    case 2
        % Compare with worst-case performance achieved by H2 filter
        figure(fh)
        [gh2,PIh2] = robfiltwcg(Pu,Fh2,nmeas,omega);
        hold on;
        ph=semilogx(omega,gh2,'r--');
        set(ph,'LineWidth',3);
        legend('Optimal Lower Bnd','H2 Filter',0);
        
        % Overlay gain of reference model
        [sv,w]=sigma(Fref,omega);
        ph2 = semilogx(omega,sv(1,:),'g--');
        set(ph2,'LineWidth',3);
        
    case 3
        % Compare with worst-case performance achieved by geometric filter
        figure(fh)
        [ggeo,PIgeo] = robfiltwcg(Pu,Fgeo,nmeas,omega);
        hold on;
        ph=semilogx(omega,ggeo,'r--');
        set(ph,'LineWidth',3);
        legend('Optimal Lower Bnd','Geometric Filter',0);
        
        % Overlay gain of reference model
        [sv,w]=sigma(Fref,omega);
        ph2 = semilogx(omega,sv(1,:),'g--');
        set(ph2,'LineWidth',3);
end       

return
% Verify that IQC condition is satisfied at all frequencies
if 0 
    Pufr = frd(Pu,omega);
    [Pfr,Delta] = lftdata(Pufr.ResponseData);
    [Nw,Nv]=size(Delta);
    lmax = zeros(nomega,1);
    for i1=1:nomega
        % Get blocks of P
        P1 = Pfr(1:Nv,:,i1);
        P1I = [P1; eye(Nw) zeros(Nw,Nd)];
        P2 = Pfr(Nv+1:Nv+Ny,:,i1);
        P3 = Pfr(Nv+Ny+1:end,:,i1);    

        FPI = [-Fopt1(:,:,i1)*P2+P3; zeros(Nd,Nw) eye(Nd)];
        M = blkdiag( eye(Nz), -gain1(i1)^2*eye(Nd) );

        IQCLMI = P1I'*PIopt1(:,:,i1)*P1I + FPI'*M*FPI;
        lmax(i1) = max(real(eig(IQCLMI)));       
    end
    fprintf(['Max eval from IQC condition = ' num2str(max(lmax))]);
    fprintf('  (should be < 0) \n\n');
end

%------------------------------------------------------------------------
% Compute upper bound using ROBFILTSYN
%------------------------------------------------------------------------
fprintf('Beginning Frequency-Gridding Robust Filter Synthesis \n');
Fbasis = {tf(1,[1 1]) tf([1 -1],[1 2 1]), tf(1,[1 10])};
%Fbasis = {tf(1,[1 1]) tf([1 -1],[1 1])};
%Fbasis = {tf(1,[1 1])};
tic
[gopt,Fopt5,PIopt,xopt] = robfiltsyn(Pu,Fbasis,nmeas,omega);
t3=toc;
gain5ub = gopt;
fprintf('Freq. Grid Syn: L2-L2 gain = %4.3f \n',gain5ub);
fprintf('Comp time = %4.3f sec\n',t3);


% Compute worst-case performance achieved by optimal filter
figure(fh)
[ggeo,PIgeo] = robfiltwcg(Pu,Fopt5,nmeas,omega);
hold on;
ph=semilogx(omega,ggeo,'g');
set(ph,'LineWidth',3);
legend('Lower Bnd','Geometric Filt','Robust Filt',0);


% % Use WCGAIN to compute gain achieved by optimal filter
% % Should get gain5ub = gain6ub;
% GF = [-Fopt5 eye(Nz)]*Pu;
% GFfr = frd(GF,omega);
% opt = wcgopt;
% opt.RelTol  = 0.01;
% [MAXGAIN6,MAXGAINUNC6,INFO6] = wcgain(GFfr,opt);
% gain6ub = MAXGAIN6.UpperBound;
% fprintf('WCGAIN: L2-L2 gain for freq. grid filter = %4.3f \n',gain6ub);
% fprintf(' (Two gains above should be equal)\n\n');

% Plot Bode of optimal filter
figure;
bode(Fopt5,omega);


% Test Fault Decoupling of Robust Filter 
Ru = Fopt5*Gu; 
figure;
step(Ru(:,1));
figure;
step(Ru(:,2));

