%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% testminsdp.m
%
% This file is used to test the LMILab translator.
% The minimum cost and computation time are
% returned.  Computation time includes all
% time for the translator as while as solver time.
%
% All solvers are tested, so the user should
% modify this file to test only solvers of interest.
% Also, this file uses the function, 'solverpath',
% to add and remove solvers from the path.  This
% prevents solvers with overlapping function names
% from interfering with each other.
%
% 
% 6/29/2001    Pete Seiler
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test Example : min alpha
%   subject to:   P>0
%                 I-P>0
%                 alpha*I - (A'P + PA) > 0     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nstate = 10;
sys = sysrand(nstate,1,1,1);
[A,B,C,D]=unpck(sys);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve with LMILab 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
setlmis([]);
[P np Xp]=lmivar(1,[nstate 1]);
[a na Xa]=lmivar(1,[1 0]);

% P>0 
lmiterm([-1 1 1 P],1,1);

% P< I
lmiterm([2 1 1 P],1,1);
lmiterm([-2 1 1 0],eye(nstate));

% A'*P +PA < a*I
lmiterm([-3 1 1 a],1,1);
lmiterm([3 1 1 P],A',1,'s');

LMISYS=getlmis;
c = zeros(na,1);c(na)=1;

options = zeros(5,1);options(5)=1;options(1)=1e-4;
x1 =clock;
[clab,xlab] =mincx(LMISYS,c,options);
x2 =clock;tlab=etime(x2,x1);
Plab = dec2mat(LMISYS,xlab,P);
alab = dec2mat(LMISYS,xlab,a); 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve with LMILab Translator: Sedumi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
solverpath('sedumi','addsolver');
options.prt=0;
x1=clock;
[csed,xsed,Zsed,infosed] = minsdp('sedumi',LMISYS,c,options);
x2=clock;tsed = etime(x2,x1);
solverpath('sedumi','rmsolver');

% Report data
Psed = dec2mat(LMISYS,xsed,P); 
ased = dec2mat(LMISYS,xsed,a); 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve with LMILab Translator: SDPpack
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
solverpath('sdppack','addsolver');
x1=clock;
[cpack,xpack,Zpack,infopack] = minsdp('sdppack',LMISYS,c,options);
x2=clock;tpack = etime(x2,x1);
solverpath('sdppack','rmsolver');

% Report data
Ppack = dec2mat(LMISYS,xpack,P);
apack = dec2mat(LMISYS,xpack,a);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve with LMILab Translator: SDPHA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
solverpath('sdpha','addsolver');
x1=clock;
[cha,xha,Zha,infoha] = minsdp('sdpha',LMISYS,c,options);
x2=clock;tha = etime(x2,x1);
solverpath('sdpha','rmsolver');

% Report data
Pha = dec2mat(LMISYS,xha,P);
aha = dec2mat(LMISYS,xha,a);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve with LMILab Translator: SP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
solverpath('sp','addsolver');
x1=clock;
[csp,xsp,Zsp,infosp] = minsdp('sp',LMISYS,c,options);
x2=clock;tsp = etime(x2,x1);
solverpath('sp','rmsolver');

% Report data
Psp = dec2mat(LMISYS,xsp,P);
asp = dec2mat(LMISYS,xsp,a);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve with LMILab Translator: SDPT3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
solverpath('sdpt3','addsolver');
x1=clock;
[ct3,xt3,Zt3,infot3] = minsdp('sdpt3',LMISYS,c,options);
x2=clock;tt3 = etime(x2,x1);
solverpath('sdpt3','rmsolver');

% Report data
Pt3 = dec2mat(LMISYS,xt3,P);
at3 = dec2mat(LMISYS,xt3,a);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Print out optimal cost and computation time
% for each solver.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dat=[alab tlab; ased tsed; apack tpack; aha tha; asp tsp; at3 tt3];
slvr{1} = 'lmilab'; slvr{2} = 'sedumi'; slvr{3} = 'sdppack';
slvr{4} = 'sdpha'; slvr{5} = 'sp'; slvr{6}= 'sdpt3';
fprintf('\nSolver \t Min Cost \t Computation Time \n');
for i1 = 1:6
  fprintf(slvr{i1});
  fprintf('\t %0.3f',dat(i1,1));
  fprintf('\t\t %0.3f \n',dat(i1,2));
end
fprintf('\n\n');





